# -*- coding: utf-8 -*-
""" Covid coronavirus statistics based on John Hopkins University statistics

"""
import requests

from covid import config
from covid.john_hopkins.models import CountryModel, CovidModel

BASE_URL = "https://services1.arcgis.com"
PATH = (
    "/0MSEUqKaxRlEPj5g/arcgis/rest/services/ncov_cases/FeatureServer/2/query"
)
URL = BASE_URL + PATH
SOURCE = config.JOHN_HOPKINS


class Covid:
    """Class handells all functionality"""

    def __init__(self):
        self.source = SOURCE

    @staticmethod
    def __get_total_cases_by_country_id(object_id: str) -> dict:
        """Method formats and encodes the URL for a specific country information regarding Covid

        Args:
            country (str): Country name e.g. "sweden"

        Returns:
            dict: Country related information regarding Coronavirus
            example:
                    {
                        'country': 'Sweden',
                        'confirmed': 355,
                        'active': 334,
                        'deaths': 0,
                        'recovered': 1,
                        'latitude': 63.0,
                        'longitude': 16.0,
                        'last_update': 1583893094000
                    }
        """

        params = dict(
            f="json",
            where=f"OBJECTID = {object_id}",
            returnGeometry="false",
            spatialRel="esriSpatialRelIntersects",
            outFields="*",
            resultOffset="0",
            resultRecordCount="1",
            cacheHint="true",
        )

        response = requests.get(URL, params=params).json()
        try:
            return response["features"][0]["attributes"]
        except KeyError:
            raise Exception(response)

    def __get_total_by_case(self, case: str) -> int:
        """Method fetchs the total value of a specific case (Deaths, Confirmed cases and Recovered cases)

        Args:
            case (str): cases = "Deaths", "Confirmed" and "Recovered"

        Returns:
            str: Total value
        """
        params = dict(
            f="json",
            where="Confirmed > 0",
            returnGeometry="false",
            spatialRel="esriSpatialRelIntersects",
            outFields="*",
            outStatistics=str(
                [
                    {
                        "statisticType": "sum",
                        "onStatisticField": f"{case}",
                        "outStatisticFieldName": "value",
                    }
                ]
            ),
            cacheHint="true",
        )
        response = requests.get(URL, params=params).json()
        try:
            return response["features"][0]["attributes"]["value"]
        except KeyError:
            raise Exception(response)

    def __get_all_cases(self) -> list:
        """Method fetches all data related to Covid

        Returns:
            list: list of Covid data
                example:
                        [
                            {
                                'id': '53',
                                'country': 'China',
                                'confirmed': 81020,
                                'active': 9960,
                                'deaths': 3217,
                                'recovered': 67843,
                                'latitude': 30.5928,
                                'longitude': 114.3055,
                                'last_update': 1584097775000
                            }
        """
        params = dict(
            f="json",
            where="Confirmed > 0",
            returnGeometry="false",
            spatialRel="esriSpatialRelIntersects",
            outFields="*",
            orderByFields="Confirmed desc",
            resultOffset="0",
            resultRecordCount="200",
            cacheHint="true",
        )
        response = requests.get(URL, params=params).json()
        try:
            return response["features"]
        except KeyError:
            raise Exception(response)

    def get_data(self) -> list:
        """Method fetches all data related to Covid"""

        cases = self.__get_all_cases()
        return [CovidModel(**case["attributes"]).dict() for case in cases]

    def get_total_deaths(self) -> int:
        """Method fetches and returns total deaths number

        Returns:
            int: Total number of deaths at this time
        """
        return self.__get_total_by_case("Deaths")

    def get_total_confirmed_cases(self) -> int:
        """Method fetches and returns the total number of confirmed cases

        Returns:
            int: Total number of confirmed cases at this time
        """
        return self.__get_total_by_case("Confirmed")

    def list_countries(self) -> list:
        """Method returns the names of all countries available, so that it can be used when
        querying status by a specific country

        Returns:
            list[str]: list of country names
        """
        cases = self.__get_all_cases()
        return [CountryModel(**case["attributes"]).dict() for case in cases]

    def get_status_by_country_id(self, country_id) -> dict:
        """Method fetches and returns specific country information related to coronavirus

        Args:
            country (str):  Country name e.g. "sweden"

        Returns:
            dict: Country related information regarding Coronavirus
            example:
                    {
                        'country': 'Sweden',
                        'confirmed': 355,
                        'active': 334,
                        'deaths': 0,
                        'recovered': 1,
                        'latitude': 63.0,
                        'longitude': 16.0,
                        'last_update': 1583893094000
                    }
        """

        case = self.__get_total_cases_by_country_id(country_id)
        return CovidModel(**case).dict()

    def get_status_by_country_name(self, country_name) -> dict:
        """Method fetches and returns specific country information related to coronavirus

        Args:
            country (str):  Country name e.g. "sweden"

        Returns:
            dict: Country related information regarding Coronavirus
            example:
                    {
                        'country': 'Sweden',
                        'confirmed': 355,
                        'active': 334,
                        'deaths': 0,
                        'recovered': 1,
                        'latitude': 63.0,
                        'longitude': 16.0,
                        'last_update': 1583893094000
                    }
        """

        country = filter(
            lambda country: country["name"].lower() == country_name.lower(),
            self.list_countries(),
        )
        try:
            country = next(country)
        except StopIteration:
            raise ValueError(
                f"There is no country called '{country_name}', to check available country names use `list_countries()`"
            )
        case = self.__get_total_cases_by_country_id(country["id"])
        return CovidModel(**case).dict()



import typer
from covid import Covid
from pprint import pprint
from enum import Enum


class Options(str, Enum):
    ACTIVE = "active"
    CONFIRMED = "confirmed"
    RECOVERED = "recovered"
    DEATHS = "deaths"


app = typer.Typer()


@app.command()
def get_data(
    source: str = typer.Option(
        "john_hopkins",
        "-s",
        "--source",
        help="select source, 'john_hopkins' or  'worldometers'",
        show_default=True,
    ),
    country: str = typer.Option(
        None,
        "--country",
        "-c",
        help="get stats by country name, 'sweden' or 'italy'",
    ),
    option: Options = typer.Option(
        None,
        "--option",
        "-o",
        help="get total stats. options 'active', 'confimed', 'recovered' or 'deaths'",
    ),
    list_countries: bool = typer.Option(
        None,
        "--list-countries",
        "-l",
        help="get a list of country name depending on a source",
        is_flag=True,
    ),
):
    """Get coronavirus information"""

    typer.echo(typer.style(f"fetching data from  '{source}'", fg="green"))
    covid = Covid(source)

    if list_countries:
        data = covid.list_countries()
        pprint(data)
        return

    if option and country:
        data = covid.get_status_by_country_name(country)
        typer.echo(
            typer.style(
                f"total {option} in {country}: {data[option]}", fg="yellow"
            )
        )
        return

    if country:
        data = covid.get_status_by_country_name(country)
        pprint(data)
        return
    elif option:
        if option == Options.ACTIVE:
            data = covid.get_total_active_cases()
            typer.echo(typer.style(f"total active cases: {data}", fg="yellow"))
            return
        if option == Options.CONFIRMED:
            data = covid.get_total_confirmed_cases()
            typer.echo(
                typer.style(f"total confirmed cases: {data}", fg="yellow")
            )
            return
        if option == Options.RECOVERED:
            data = covid.get_total_recovered()
            typer.echo(
                typer.style(f"total recovered cases: {data}", fg="yellow")
            )
            return
        if option == Options.DEATHS:
            data = covid.get_total_deaths()
            typer.echo(typer.style(f"total deaths: {data}", fg="yellow"))
            return

        typer.echo(
            typer.style(
                f"Total option argument '{option}' is not allowed", fg="red"
            )
        )
        return

    data = covid.get_data()
    pprint(data)
    return


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
""" Covid coronavirus statistics based on John Hopkins University statistics

"""
from decimal import Decimal

from pydantic import BaseModel, Field


class CovidModel(BaseModel):
    """Dataclass acts as a Model for Covid data"""

    country: str = Field(..., alias="Country,Other")
    total_cases: int = Field(0, alias="TotalCases")
    confirmed: int = Field(0, alias="TotalCases")
    new_cases: int = Field(0, alias="NewCases")
    deaths: int = Field(0, alias="TotalDeaths")
    new_deaths: int = Field(0, alias="NewDeaths")
    recovered: int = Field(0, alias="TotalRecovered")
    active: int = Field(0, alias="ActiveCases")
    active_cases: int = Field(0, alias="ActiveCases")
    critical: int = Field(0, alias="Serious,Critical")
    total_tests: int = Field(0, alias="TotalTests")
    total_tests_per_million: Decimal = Field(Decimal(0), alias="Tests/1M pop")
    total_cases_per_million: Decimal = Field(
        Decimal(0), alias="TotCases/1M pop"
    )
    total_deaths_per_million: Decimal = Field(
        Decimal(0), alias="Deaths/1M pop"
    )
    population: Decimal = Field(Decimal(0), alias="Population")



# -*- coding: utf-8 -*-
""" Covid coronavirus statistics based on John Hopkins University statistics

"""
from pydantic import BaseModel, Field


class CovidModel(BaseModel):
    """Dataclass acts as a Model for Covid data"""

    id: str = Field(..., alias="OBJECTID")
    country: str = Field(..., alias="Country_Region")
    confirmed: int = Field(None, alias="Confirmed")
    active: int = Field(None, alias="Active")
    deaths: int = Field(None, alias="Deaths")
    recovered: int = Field(None, alias="Recovered")
    latitude: float = Field(None, alias="Lat")
    longitude: float = Field(None, alias="Long_")
    last_update: int = Field(None, alias="Last_Update")


class CountryModel(BaseModel):
    """Dataclass acts as a Model for Countries data"""

    id: str = Field(..., alias="OBJECTID")
    name: str = Field(..., alias="Country_Region")



# -*- coding: utf-8 -*-
from covid import config
from covid.john_hopkins import Covid as JohnHopkinsCovid
from covid.worldometers import Covid as WorldometersCovid

__author__ = "Ahmed Nafies <ahmed.nafies@gmail.com>"
__copyright__ = "Copyright 2020, Ahmed Nafies"
__license__ = "MIT"
__version__ = "2.5.2"

services = {
    "john_hopkins": JohnHopkinsCovid,
    "worldometers": WorldometersCovid,
}


def Covid(source=config.JOHN_HOPKINS):
    try:
        service = services[source]
        return service()
    except KeyError:
        raise ValueError(f"Allowed sources are {', '.join(config.SOURCES)}")
